/******************************************************************************

 fdd3000.c: The FDD3000 disk station emulator  
 
 This file is part of FDD3000 Emulator version 0.2.0

 Copyright (C) 2018 by Sławomir Szczyrba <sszczyrba@gmail.com>

 FDD300Emu is free software: you can redistribute it and/or modify it under
 the terms of the GNU Lesser General Public License as published by the Free
 Software Foundation, either version 2 of the License, or (at your option) any
 later version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 details.

 You should have received a copy of the GNU Lesser General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/lgpl.html>

******************************************************************************/

#include <stdint.h>
#include <string.h>
#include <stdio.h>

#include "fdd3boot.h"
#include "fdd3000.h"
#include "z80free/Z80free.h"

static Z80FREE cpuctx, *cpu = &cpuctx;
static uint8_t fddRAM[65536];
static int paged, drq;
static uint8_t fdd2ti, ti2fdd;
static int counter;


void
wrTI2FDD( uint8_t val )
{
  ti2fdd = val;
}

uint8_t
rdFDD2TI( void )
{
  return fdd2ti;
}

void
fddSetFDCInt( uint8_t val )
{
  Z80free_INT( cpu, val );
}

void
fddClrFDCInt( void )
{
  Z80free_INTserved( cpu );
}

void
fddSetDRQ( void )
{
  drq = 128;
}

void
fddClrDRQ( void )
{
  drq = 0;
}

void
fddReset( void )
{
  paged = 1;
  drq   = 0;
  Z80free_reset( cpu );
  counter = 0;
}

int
fddCpuStep( void )
{
  int tst;
  tst = Z80free_step( cpu );
  counter++;
  if( counter >= 5 ) {
    tst += Z80free_step( cpu ); counter = 0;
  }
  return tst;
}

int
fddCpuUStep( void )
{
  int tst;
  tst = Z80free_ustep( cpu );
  counter++;
  if( counter >= 5 ) {
    tst += Z80free_ustep( cpu ); counter = 0;
  }
  return tst;
}

byte
Z80free_Rd( word Addr )
{
  if( paged && ( Addr < 2048 ) ) {
    return fddROM[Addr];
  } else return fddRAM[Addr];
}

void
Z80free_Wr( word Addr, byte Value )
{
  if( ( !paged ) || ( Addr > 2048 ) ) {
    fddRAM[Addr] = Value;
  }
}

void
writeConfig( uint8_t value )
{
  uint8_t val = ( value ^ 0xff ) & 0b10111111;
  setDriveCfg( val );
  if( !( value & 0b01000000 ) ) {
    paged = 1;
  } else {
    paged = 0;
  }
}

/* ;;;  WD1770
   ; ------
   ; C0 - status/command
   ; C1 - track
   ; C2 - sector
   ; C3 - data */

void
writeFddPort( uint16_t port, uint8_t value )
{
  switch( port ) {
  case 0: wrCommandReg( value );   break;
  case 1: wrTrackReg( value );     break;
  case 2: wrSectorReg( value );    break;
  case 3: wrDataReg( value );      break;
  }
}

void
Z80free_Out( word Port, byte Value )
{
  int ioport = ( Port & 0xff ) >> 5;
  switch( ioport ) {
  case 0:
  case 2:
  case 3:
  case 4:
  case 5: break;       /* serial ports are not emulated yet */
  case 1: fdd2ti = Value;  break;
  case 6: writeFddPort( Port & 0x0f, Value ); break;
  case 7: writeConfig( Value ); break;
  }
}

uint8_t
readFddPort( uint16_t port )
{
  uint8_t val = 0;
  switch( port ) {
  case 0: val = rdStatusReg(); break;
  case 1: val = rdTrackReg();  break;
  case 2: val = rdSectorReg(); break;
  case 3: val = rdDataReg();   break;
  }
  return val;
}

byte
Z80free_In( word Port )
{
  int ioport = ( Port & 0xff ) >> 5;
  switch( ioport ) {
  case 0: return 0xff;
  case 1: return ti2fdd | drq;
  case 2:
  case 3:
  case 4:
  case 5: return 0xff;        /* serial ports are not emulated yet */
  case 6: return readFddPort( Port & 0x0f );
  case 7: return 0xff;        /* write only */
  }
  return 0;
}
